<?php

#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

# $Id: bibsonomy_helper.php,v 1.3 2008-05-11 14:30:17 cschenk Exp $

require_once('bibsonomy_def.php');
require_once('bibsonomy_model.php');

#
# Holds the status and a description after an interaction with BibSonomy.
#
class BibSonomyStatus {
	private $code, $description;
	public function __construct($code = STATUS_UNDEF, $description = '') {
		$this->code = $code;
		$this->description = $description;
	}
	public function getCode() { return $this->code; }
	public function getDescription() { return $this->description; }
}

#
# Checks whether the given resourceType is valid
#
function isValidResourceType($resourceType) {
	if (strtolower($resourceType) == 'bookmark' or strtolower($resourceType) == 'bibtex') {
		return true;
	}
	return false;
}

#
# Almos the same as isValidResourceType but throws an Exception
#
function assertValidResourceType($resourceType) {
	if (isValidResourceType($resourceType) == false) {
		throw new Exception('Unknown resourcetype "' + $resourceType + '"');
	}
}

#
# Converts a string to an array of Tag objects.
#
function string2Tags($string) {
	return array2Tags(preg_split('/ /', $string));
}

#
# Converts an array of (hopefully) strings to an array of Tag objects.
#
function array2Tags(array $array) {
	$tags = array();
	foreach ($array as $value) {
		if (empty($value)) continue;
		$tag = sanitizeTag($value);
		$tags[] = new Tag($tag);
	}
	return $tags;
}

#
# Some tag improvement.
#
function sanitizeTag($tag) {
	return str_replace(' ', '_', $tag);
}

function removeBibTexSpecialChars($string) {
	$string = str_replace('{', '', $string);
	$string = str_replace('}', '', $string);
	return $string;
}

?>
